#include <iostream>
#include <vector>
#include <curl/curl.h>
#include "histogram.h"
#include "histogram_internal.h"
#include "text.h"
#include "svg.h"
#include <sstream>
#include <string>

using namespace std;

    struct Input {
    size_t n;
    vector<double>A;
    size_t bin{};
    };

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx)
{
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input input_data(istream& tin, bool prompt)
{
    Input in;
    size_t n;

    if (prompt)
    {
        cout<<"Marks: ";
    }
    tin>>in.n;
    in.A.resize(in.n);

    for (size_t i=0; i<in.n; i++)
    {
        cout<<"A["<<i<<"]=";
        tin>>in.A[i];
    }

    if (prompt)
    {
       cout<<"Rows: ";
    }
    tin>>in.bin;
    return in;
}

Input download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl)
    {

        CURLcode res;
        double connect;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != 0)
        {
            cerr << curl_easy_strerror(res);
            exit(1);
        }
        else
            cerr<<"No errors (code "<<res<<")"<<'\n';
        if(CURLE_OK == res)
        {
            res = curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &connect);
            if(CURLE_OK == res)
            {
                cerr<<"Connecting time: "<<connect<<"sec";
            }
        }
        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    Input in;
    CURLcode res;
    if (argc > 1)
        in = download(argv[1]);
    else
        in = input_data(cin, true);
    size_t n;
    auto B = make_histogram(in.A, in.bin);
    show_histogram_svg(B);
    return 0;
}
